<?php

namespace Drupal\languagefield\Hook;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Hook\Attribute\Hook;
use Drupal\Core\Render\BubbleableMetadata;
use Drupal\field\FieldStorageConfigInterface;
use Drupal\languagefield\Entity\CustomLanguageManager;

/**
 * Contains Field and Help hooks.
 *
 * Class is declared as a service in services.yml file.
 *
 * @see https://drupalize.me/blog/drupal-111-adds-hooks-classes-history-how-and-tutorials-weve-updated
 */
class CustomLanguageHooks {

  /**
   * Implements hook_field_formatter_info_alter().
   *
   * The module reuses the formatters defined in list.module.
   */
  #[Hook('field_formatter_info_alter')]
  public function fieldFormatterInfoAlter(&$info) {
    $info['list_default']['field_types'][] = 'language_field';
    $info['list_key']['field_types'][] = 'language_field';
  }

  /**
   * Implements hook_field_widget_info_alter().
   *
   * The module does not implement widgets of its own, but reuses the
   * widgets defined in options.module.
   *
   * @see workflow_options_list()
   */
  #[Hook('field_widget_info_alter')]
  public function fieldWidgetInfoAlter(&$info) {
    $info['options_buttons']['field_types'][] = 'language_field';
    $info['options_select']['field_types'][] = 'language_field';
  }

  /**
   * Implements hook_token_info().
   */
  #[Hook('token_info')]
  public function tokenInfo() {
    $tokens = [];

    if (!\Drupal::hasService('token.entity_mapper')) {
      return $tokens;
    }

    foreach (\Drupal::entityTypeManager()->getDefinitions() as $entity_type_id => $entity_type) {
      if (!$entity_type->entityClassImplements(ContentEntityInterface::class)) {
        continue;
      }
      $token_type = \Drupal::service('token.entity_mapper')->getTokenTypeForEntityType($entity_type_id);
      if (empty($token_type)) {
        continue;
      }

      // Add [entity:language_field] field tokens if they do not already exist.
      $fields = \Drupal::service('entity_field.manager')->getFieldStorageDefinitions($entity_type_id);
      foreach ($fields as $field_name => $field) {
        /** @var \Drupal\Core\Field\FieldStorageDefinitionInterface $field */
        if ($field->getType() != 'language_field') {
          continue;
        }

        // Language tokens.
        $tokens["{$token_type}-{$field_name}"]['language'] = [
          'name' => t('Language'),
          'description' => t('Tokens related to language field.'),
          // 'module' => 'languagefield',
          // 'needs-data' => 'language',
        ];

        $tokens["{$token_type}-{$field_name}"]['name'] = [
          'name' => t('Language name'),
          'description' => t('The language name.'),
          // 'module' => 'languagefield',
        ];
        $tokens["{$token_type}-{$field_name}"]['langcode'] = [
          'name' => t('Language code'),
          'description' => t('The ISO 639 language code.'),
          // 'module' => 'languagefield',
        ];
      }
    }

    return [
      'tokens' => $tokens,
    ];
  }

  /**
   * Implements hook_tokens().
   */
  #[Hook('tokens')]
  public function tokens($type, $tokens, array $data, array $options, BubbleableMetadata $bubbleable_metadata) {
    $replacements = [];

    if (empty($data['field_property'])) {
      return $replacements;
    }
    if (!isset($data['field_name']) || !isset($data[$data['field_name']])) {
      return $replacements;
    }

    $token_service = \Drupal::token();
    $url_options = ['absolute' => TRUE];
    $language_manager = \Drupal::languageManager();
    if (isset($options['langcode'])) {
      $url_options['language'] = $language_manager->getLanguage($options['langcode']);
      $langcode = $options['langcode'];
    }
    else {
      $langcode = $language_manager->getCurrentLanguage()->getId();
    }

    foreach ($tokens as $token => $original) {
      $delta = 0;
      $parts = explode(':', $token);
      if (is_numeric($parts[0])) {
        if (count($parts) > 1) {
          $delta = $parts[0];
          $property_name = $parts[1];
        }
        else {
          continue;
        }
      }
      else {
        $property_name = $parts[0];
      }
      $field_item = $data[$data['field_name']][$delta];
      $langcode = is_object($field_item) ? $field_item->value : NULL;
      if ($langcode) {
        $language = CustomLanguageManager::createFromLangcode($langcode);
      }

      if (!empty($language)) {
        foreach ($tokens as $name => $original) {
          switch ($name) {
            case 'name':
              $replacements[$original] = $language->getName();
              break;

            case 'langcode':
              // Not used, use field:value.
              $replacements[$original] = $langcode;
              break;

          }
        }
      }
    }
    return $replacements;
  }

  /**
   * Implements hook_field_views_data().
   */
  #[Hook('field_views_data')]
  public function fieldViewsData(FieldStorageConfigInterface $field_storage) {
    $data = (version_compare(\Drupal::VERSION, '11.2') >= 0)
      ? \Drupal::service('views.field_data_provider')
        ->defaultFieldImplementation($field_storage)
      : views_field_default_views_data($field_storage);

    foreach ($data as $table_name => $table_data) {
      $name = $field_storage->getName();
      $data[$table_name]["{$name}_value"]['filter']['id'] = 'languagefield';
    }

    return $data;
  }

}
